import requests

from handlers.inderrhandler import handle_status_code, handle_log

# The way of using this class is simple
# Create instance of an object class
# The object is bound to one market pair e.g. BTC/USDT
# Happily read the indicators!
# Big win =]
class Client:
    def __init__(self, api_key, create_logs=False):
        self.create_logs = create_logs
        self.headers = {
            'X-RapidAPI-Key': api_key,
            'X-RapidAPI-Host': 'crypto-indicators-rest.p.rapidapi.com'
        }
        # This api seems to work well.
        # There are plenty of indicators,
        # Which may be helpful to users.
        # Limit - 2000 requests per month for free.
        # I will add all available indicators ASAP.
        self.url = 'https://crypto-indicators-rest.p.rapidapi.com/'

    def rsi(self, market, timeframe='1d', period=14):
        url = self.url + 'rsi'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def stochastic_rsi(self,
                       market,
                       timeframe='1d',
                       rsiperiod=14,
                       stochasticperiod=14,
                       kperiod=3,
                       dperiod=3):
        url = self.url + 'stochasticrsi'
        data = {
            'market': market,
            'timeframe': timeframe,
            'rsiPeriod': rsiperiod,
            'stochasticPeriod': stochasticperiod,
            'kPeriod': kperiod,
            'dPeriod': dperiod
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def macd(self,
             market,
             timeframe='1d',
             fastperiod=12,
             slowperiod=26,
             signalperiod=9,
             simplemaoscillator=False,
             simplemasignal=False):
        url = self.url + 'macd'
        data = {
            'market': market,
            'timeframe': timeframe,
            'fastPeriod': fastperiod,
            'slowPeriod': slowperiod,
            'signalPeriod': signalperiod,
            'SimpleMAOscillator': simplemaoscillator,
            'SimpleMASignal': simplemasignal
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def adx(self, market, timeframe='1d', period=14):
        url = self.url + 'adx'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def atr(self, market, timeframe='1d', period=14):
        url = self.url + 'atr'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def awesome_oscillator(self, market, timeframe='1d', fastperiod=5, slowperiod=34):
        url = self.url + 'awesomeOscillator'
        data = {
            'market': market,
            'timeframe': timeframe,
            'fastPeriod': fastperiod,
            'slowPeriod': slowperiod
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def bollinger_bands(self, market, timeframe='1d', period=14):
        url = self.url + 'bollingerBands'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def cci(self, market, timeframe='1d', period=20):
        url = self.url + 'cci'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def force_index(self, market, timeframe='1d', period=1):
        url = self.url + 'forceIndex'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def kst(self,
            market,
            timeframe='1d',
            rocper1=10,
            rocper2=15,
            rocper3=20,
            rocper4=30,
            smarocper1=10,
            smarocper2=10,
            smarocper3=10,
            smarocper4=15,
            signalperiod=9):
        url = self.url + 'kst'
        data = {
            'market': market,
            'timeframe': timeframe,
            'ROCPer1': rocper1,
            'ROCPer2': rocper2,
            'ROCPer3': rocper3,
            'ROCPer4': rocper4,
            'SMAROCPer1': smarocper1,
            'SMAROCPer2': smarocper2,
            'SMAROCPer3': smarocper3,
            'SMAROCPer4': smarocper4,
            'signalPeriod': signalperiod
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def money_flow_index(self, market, timeframe='1d', period=14):
        url = self.url + 'moneyFlowIndex'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def on_balance_volume(self, market, timeframe='1d'):
        url = self.url + 'onBalanceVolume'
        data = {
            'market': market,
            'timeframe': timeframe
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def parabolic_stop_and_reverse(self, market, timeframe='1d', step=0.2, max_=0.2):
        url = self.url + 'psar'
        data = {
            'market': market,
            'timeframe': timeframe,
            'step': step,
            'max': max_
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            handle_status_code(response)
            return None

    def sma(self, market, timeframe='1d', period=9):
        url = self.url + 'sma'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def stochastic_oscillator(self, market, timeframe='1d', period=14, signalperiod=3):
        url = self.url + 'stochasticOscillator'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period,
            'signalPeriod': signalperiod
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def trix(self, market, timeframe='1d', period=18):
        url = self.url + 'trix'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def volume_profile(self, market, timeframe='1d', nofbars=14):
        url = self.url + 'volumeProfile'
        data = {
            'market': market,
            'timeframe': timeframe,
            'noOfBars': nofbars
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def ema(self, market, timeframe='1d', period=9):
        url = self.url + 'ema'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def wma(self, market, timeframe='1d', period=9):
        url = self.url + 'wma'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def wema(self, market, timeframe='1d', period=7):
        url = self.url + 'wema'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def williams_r(self, market, timeframe='1d', period=14):
        url = self.url + 'williamsR'
        data = {
            'market': market,
            'timeframe': timeframe,
            'period': period
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None

    def ichimoku_cloud(self,
                       market,
                       timeframe='1d',
                       conversionperiod=9,
                       baseperiod=26,
                       spanperiod=52,
                       displacement=26):
        url = self.url + 'ichimokuCloud'
        data = {
            'market': market,
            'timeframe': timeframe,
            'conversionPeriod': conversionperiod,
            'basePeriod': baseperiod,
            'spanPeriod': spanperiod,
            'displacement': displacement
        }
        response = requests.get(url, headers=self.headers, params=data)
        if response.status_code == 200:
            return response.json()
        else:
            if self.create_logs:
                handle_status_code(response)
            return None


if __name__ == '__main__':
    pass




from datetime import datetime
import os


def handle_log(func):
    def wrapped(*args, **kwargs):
        msg = func(*args, **kwargs)
        date = datetime.today().strftime('%d/%m/%Y %H:%M:%S')
        with open(f'{os.getcwd()}/logs.txt', 'a') as f:
            f.write(f'{date:7s} {msg}\n')
    return wrapped


@handle_log
def handle_status_code(response):
    status_code = response.status_code
    if status_code == 400:
        return "BAD REQUEST!"
    elif status_code == 401:
        return "UNAUTHORIZED! ENTER VALID API KEY!"
    elif status_code == 402:
        return "PAYMENT REQUIRED!"
    elif status_code == 403:
        return "FORBIDDEN! ENTER VALID API KEY!"
    elif status_code == 404:
        return "NOT FOUND!"
    elif status_code == 500:
        return "SERVER ERROR!"
    else:
        return "UNKNOWN ERROR!"



from setuptools import setup, find_packages

VERSION = '0.0.1'
DESCRIPTION = 'IndicatorAPI.com wrapper'
LONG_DESCRIPTION = 'A package that allows developers to use indicatorapi quickly.'

# Setting up
setup(
    name="pyIndicatorAPI",
    version=VERSION,
    author="dreistein33",
    author_email="kjuchnowski@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['datetime', 'os', 'requests'],
    keywords=['python', 'indicatorapi', 'api', 'crypto'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


import time

from ..indicators import Client

api = ''


class TestClient:

    def test_rsi(self):
        client = Client(api)
        response = client.rsi('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_stochastic_rsi(self):
        client = Client(api)
        response = client.stochastic_rsi('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_macd(self):
        client = Client(api)
        response = client.macd('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_adx(self):
        client = Client(api)
        response = client.adx('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_atr(self):
        client = Client(api)
        response = client.adx('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_awesome_oscillator(self):
        client = Client(api)
        response = client.awesome_oscillator('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_bollinger_bands(self):
        client = Client(api)
        response = client.bollinger_bands('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_cci(self):
        client = Client(api)
        response = client.cci('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_force_index(self):
        client = Client(api)
        response = client.force_index('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_kst(self):
        client = Client(api)
        response = client.kst('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_money_flow_index(self):
        client = Client(api)
        response = client.money_flow_index('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_on_balance_volume(self):
        client = Client(api)
        response = client.on_balance_volume('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_parabolic_stop_and_reverse(self):
        client = Client(api)
        response = client.parabolic_stop_and_reverse('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_sma(self):
        client = Client(api)
        response = client.sma('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_stochastic_oscillator(self):
        client = Client(api)
        response = client.stochastic_oscillator('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_trix(self):
        client = Client(api)
        response = client.trix('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_volume_profile(self):
        client = Client(api)
        response = client.volume_profile('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_ema(self):
        client = Client(api)
        response = client.ema('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_wma(self):
        client = Client(api)
        response = client.wma('BTC/USDT')
        time.sleep(2)
        assert response is not None
        
    def test_wema(self):
        client = Client(api)
        response = client.wema('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_williams_r(self):
        client = Client(api)
        response = client.williams_r('BTC/USDT')
        time.sleep(2)
        assert response is not None

    def test_ichimoku_cloud(self):
        client = Client(api)
        response = client.ichimoku_cloud('BTC/USDT')
        time.sleep(2)
        assert response is not None
